from __future__ import absolute_import
import json
import argparse

#Metodo responsible for format output to user
def print_pretty(command_return):
    transform_json = json.dumps(command_return, indent=2)
    print(transform_json)
    

class main():
    from . import PyJboss
     
    # main command
    description = 'This package can be used to call the jbossapi by http protocol and return values'
    main_parse = argparse.ArgumentParser(prog='jbosspy',
                                         description=description)

    main_parse.add_argument('-u', '--user',
                            help="Jboss user with administrator permissions.")
    main_parse.add_argument('-p', '--password',
                            help="Jboss user password.")
    main_parse.add_argument('-c', '--controller',
                            help="IP or name of Jboss host controller, or domain controller.")
    main_parse.add_argument('--host',
                            help="A host from domain. This option is only necessary if your jboss is running in domain mode.")
    main_parse.add_argument('--server',
                            help="A server from the host. This option is only necessary if your jboss is running in domain mode.")
    main_parse.add_argument("-v",
                            "--verbose",
                            action="store_true",
                            required=False)

    main_subparse = main_parse.add_subparsers(title="Commands", dest='command')
    main_subparse.required = True

    # parse ejb command
    ejb_parse = main_subparse.add_parser(name="ejb",
                                         help='Manage ejb resources',
                                         description="Call the ejb resources")
    ejb_parse.add_argument("--thread-pool-name",
                           required=True,
                           help="Ejb thread pool name")
    ejb_parse.set_defaults(command="ejb")

    # parse message command
    message_parse = main_subparse.add_parser(
        name="message",
        help="Manage message resources",
        description="Call the message ressources")
    message_parse.add_argument('--queue-name', help="Queue name")
    message_parse.add_argument('--topic-name', help="Topic name")
    message_parse.add_argument('--list',
                               choices=['queue', 'topic'],
                               help="Return a list of queues or topics")
    message_parse.add_argument(
        '--message-server',
        required=True,
        help="Name of the server defined in the message resource")
    message_parse.set_defaults(command="message")

    # parse datasource command
    datasource_parse = main_subparse.add_parser(
        name="datasource",
        help="Manage datasource resources",
        description="Get information about datasources")
    datasource_parse.add_argument(
        '--datasource',
        help="Show information about an datasource, important: to obtain more information enable datasource statistic"
    )
    datasource_parse.add_argument(
        '--xa-datasource',
        help="Return information about an xa-datasource, important: to obtain more information enable datasource statistic"
    )
    datasource_parse.add_argument('--list',
                                  help="Obtain a list of datasources", choices=['datasource', 'xa-datasource'])
    datasource_parse.set_defaults(command="datasource")

    # parse utils command
    utils_parse = main_subparse.add_parser(
        name='utils',
        help='Get information aboute many resources in general'
    )
    utils_parse.add_argument('--get-memory-info',
                             help='Get jvm memory in use',
                             action="store_true",
                             required=False
                             )
    utils_parse.set_defaults(command="utils")

    # return main_parse
    parse_return = main_parse.parse_args()
    
    #create a PyJboss object
    obJboss = PyJboss(controller=parse_return.controller,
                      user=parse_return.user,
                      password=parse_return.password,
                      host=parse_return.host,
                      server=parse_return.server)

    # process command ejb
    if parse_return.command == 'ejb':
        print_pretty(command_return=obJboss.ejb.get_thread_pool(thread_pool_name=parse_return.thread_pool_name))
    # process command message
    elif parse_return.command == 'message':
        if parse_return.queue_name:
            print_pretty(
                obJboss.message.get(name_resource=parse_return.queue_name,
                                    message_server=parse_return.message_server,
                                    type_message="queue"))

        elif parse_return.topic_name:
            print_pretty(
                obJboss.message.get(name_resource=parse_return.topic_name,
                                    message_server=parse_return.message_server,
                                    type_message="topic"))
    # process command datasource
    elif parse_return.command == 'datasource':
        if parse_return.list:
            print_pretty(obJboss.datasource.list(datasource_type=parse_return.list))
        elif parse_return.datasource:
            print_pretty(
                obJboss.datasource.get(
                    datasource_type='datasource',
                    datasource_name=parse_return.datasource))
    # process command utils
    elif parse_return.command == 'utils':
        if parse_return.get_memory_info == True:
            print_pretty(obJboss.utils.get_memory_info())



from ..transport import Transport
from .datasource import DatasourcePy
from .message import MessagePy
from .ejb import EjbPy
from .utils import UtilsPy

import logging


class PyJboss(object):
    def __init__(self, controller, user, password, host=None, server=None):
        self.controller = controller
        self.user = user
        self.password = password
        self.host = host
        self.server = server

        self.transport = Transport(user=self.user,
                                   password=self.password,
                                   controller=self.controller,
                                   host=self.host,
                                   server=self.server)
        
        self.datasource = DatasourcePy(self)
        self.message = MessagePy(self)
        self.ejb = EjbPy(self)
        self.utils = UtilsPy(self)



from .namespacedclient import NamespacedClient


class MessagePy(NamespacedClient):
    def list(self, message_server='default', type_message='queue'):
        payload = {
            "address": [{
                "subsystem": "messaging-activemq"
            }, {
                "server": message_server
            }],
            "operation":
            "read-children-names",
            "child-type":
            "jms-" + type_message
        }
        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)

        return return_data

    def get(self,
            name_resource,
            message_server='default',
            type_message='queue'):
        payload = {
            "address": [{
                "subsystem": "messaging-activemq"
            }, {
                "server": message_server
            }, {
                "jms-" + type_message: name_resource
            }],
            "operation":
            "read-resource",
            "include-runtime":
            True
        }
        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)

        return return_data



from .namespacedclient import NamespacedClient


class EjbPy(NamespacedClient):
    '''
    Class utilized to get information about ejb resources

    '''
    def list_thread_pools(self):
        payload = {"address" : [{ "subsystem" : "ejb3" }], "operation" : "read-children-names", "child-type" : "thread-pool"}
        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)
        return return_data
    
    def get_thread_pool(self, thread_pool_name='default'):
        payload = {"address" : [{ "subsystem" : "ejb3" },{ "thread-pool" : thread_pool_name }], "operation" : "read-resource", "include-runtime" : True}
        
        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)
        return return_data


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyjboss", 
    version="0.0.2",
    license="GNU General Public License v3.0",
    author="Rafael Benites Gil",
    author_email="rafaelbgil@gmail.com",
    description="Python client for Jboss and Wildfly api",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rafaelbgil/pyjboss",
    py_modules=['pyjboss'],
    packages=setuptools.find_packages(),
    classifiers=[
	"Development Status :: 3 - Alpha",
	"Programming Language :: Python",
	"Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
	"Programming Language :: Python :: 3.0",
	"Programming Language :: Python :: 3.1",
	"Programming Language :: Python :: 3.2",
	"Programming Language :: Python :: 3.3",
	"Programming Language :: Python :: 3.4",
	"Programming Language :: Python :: 3.5",
	"Programming Language :: Python :: 3.6",
	"Programming Language :: Python :: 3.7",
	"Programming Language :: Python :: 3.8",
	"Programming Language :: Python :: 3.9",
	"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    install_requires = ['requests'],
    python_requires='>=2.7',
)



from .namespacedclient import NamespacedClient


class UtilsPy(NamespacedClient):
    def get_memory_info(self):
        payload = {"address": [{"core-service": "platform-mbean"}, {
            "type": "memory"}], "operation": "read-resource", "include-runtime": True}
        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)
        return return_data



import requests
import json

class Transport(object):
    '''
    Class used to make requests to jboss api and manage errors
    '''

    def __init__(self, user, password, controller, host=None, server=None):
        self.user = user
        self.password = password
        self.controller = "http://" + controller + ":9990/management"
        self.host = host
        self.server = server
    
    def analise_return(self, request):
        if request.status_code ==  401:
            print("user or password invalid.")
            return False
        elif not 'result' in request.json():
            print("No resources found.")
            return False
        elif request.ok and len(request.json()['result']) > 0:
            return request.json()['result']
        elif len(request.json()['result']) == 0:
            print("No resources found.")
            return False
        

    def make_request(self, method, endpoint, payload=None, params=None):
        headers = {'content-type': 'application/json'}
        authentication = requests.auth.HTTPDigestAuth(
            username=self.user, password=self.password)
        if payload is not None:
            if not self.server == None:
                payload['address'].insert(0, {"server" : self.server})
            if not self.host == None:
                payload['address'].insert(0, {"host" : self.host})
            payload = json.dumps(payload)
        if method == 'POST':
            try:
                return self.analise_return(requests.post(auth=authentication, url=endpoint, headers=headers, data=payload))
            except requests.exceptions.ConnectionError:
                print("Cannot connect to host controller url %s." % (self.controller))
                return False
            except: 
                print("Cannot obtain a valid reponse.")
                return False
        



from .namespacedclient import NamespacedClient


class DatasourcePy(NamespacedClient):
    ''' 
    Class used to management the datasources

    '''
    def normalize_datasource(self, datasource_type='datasource'):
        #Method utilized internaly only, that is reponsible to format output from datasource and xa-datasource
        if datasource_type == 'datasource':
            datasource_type = 'data-source'
        elif datasource_type == 'xa-datasource':
            datasource_type = 'xa-data-source'
        return datasource_type

    def list(self, datasource_type='datasource'):
        payload = {
            "address": [{
                "subsystem": "datasources"
            }],
            "operation": "read-children-names",
            "child-type": self.normalize_datasource(datasource_type)
        }

        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)
        return return_data

    def get(self, datasource_type, datasource_name):
        payload = {
            "address": [{
                "subsystem": "datasources"
            }, {
                self.normalize_datasource(datasource_type):
                datasource_name
            }],
            "operation":
            "read-resource",
            "include-runtime":
            True
        }
        return_data = self.transport.make_request(
            method='POST', endpoint=self.transport.controller,
            payload=payload)
        return return_data
